@@ -62,6 +62,8 @@
 #define ABOUTqual	18
 #define STATSqual	19
 #define NOSTATSqual	20
+#define ISO8601qual     21
+#define ZISO8601qual    22
 
 /******************************************************************************
 * Macro constants.
@@ -110,7 +112,7 @@ Logical CreateSkeletonTable PROTOARGs((int argC, char *argV[]));
 #if defined(__cplusplus)
 }
 #endif
-
+int iso8601;
 Logical SkeletonTableQOPs PROTOARGs((int *argC, char **argV[]));
 Logical WriteHeader PROTOARGs((char *CDFname, int varValues, char *variables));
 Logical WriteGlobalAttr PROTOARGs((void));
