@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                    Header file for CDFdump.
 *
@@ -28,25 +33,36 @@
 * Include files.
 ******************************************************************************/
 
+#include "cdflib.h"
 #include "cdftools.h"
 
 /******************************************************************************
 * Parameter/qualifier constants.
 ******************************************************************************/
 
-#define CDFPATHparm     0
+#define CDFPATHparm       0
 
-#define DUMPqual        0
-#define OUTPUTqual      1
-#define FORMATqual      2
-#define NOFORMATqual    3
-#define VARSqual        4
-#define ABOUTqual	5
-#define NOHEADERqual	6
+#define DUMPqual          0
+#define OUTPUTqual        1
+#define FORMATqual        2
+#define NOFORMATqual      3
+#define PRECISIONqual     4
+#define VARSqual          5
+#define ABOUTqual    	  6
+#define HEADERqual	  7
+#define NOHEADERqual	  8
+#define RECORDRANGEqual   9
+#define COL2ROWqual      10
+#define DETECTqual       11
+#define EPOCHVVqual      12
+#define ZISO8601qual     13
 
-#define ALLDUMP         0
-#define DATADUMP        1
-#define METADATADUMP    2
+#define ALLDUMP           0
+#define DATADUMP          1
+#define NODATADUMP        2
+#define METADATADUMP      3
+#define GLOBALDUMP        4
+#define VARIABLEDUMP      5
 
 #define DEFAULToptionDUMP ALLDUMP
 /******************************************************************************
@@ -58,21 +74,29 @@ Logical useFormat = DEFAULTformatDUMP;
 char outputCDF[CDF_PATHNAME_LEN+1] = "default";
 char dumpOption[20];
 char varsOption[513];
-Logical noheader = DEFAULTnoheaderDUMP;
+Logical header = DEFAULTheaderDUMP;
+Logical col2row = DEFAULTcol2rowDUMP;
+long startingRec, endingRec;
+Logical epochVV = DEFAULTepochVVDUMP;
+Logical ziso8601;
 #else
 extern char outputCDF[CDF_PATHNAME_LEN+1];
 extern int dump;
 extern Logical useFormat;
 extern char dumpOption[20];
 extern char varsOption[513];
-extern Logical noheader;
+extern Logical header;
+extern Logical col2row;
+extern long startingRec, endingRec;
+extern Logical epochVV;
+extern Logical ziso8601;
 #endif
 /******************************************************************************
 * Function Prototypes.
 ******************************************************************************/
 
 Logical DumpCDF PROTOARGs((int argC, char *argV[]));
-void QuitCDF PROTOARGs((char *text, CDFstatus status, char *msg));
+Logical QuitCDF PROTOARGs((char *file, char *text, CDFstatus status, char *msg));
 
 /*****************************************************************************/
 
