@@ -69,6 +69,7 @@
 #define SETTINGSSEQUENCEqual    21
 #define EPOCHRANGEqual          22
 #define RECORDRANGEqual         23
+#define ZISO8601qual            24
 
 #define FORMATopt               0
 #define NOFORMATopt             1
@@ -114,6 +115,7 @@
 #define NOHEADINGopt            41
 #define ISO8601opt              42
 #define POUNDEDHEADINGopt       43
+#define ZISO8601opt             44
 
 /******************************************************************************
 * Miscellaneous.
@@ -163,7 +165,7 @@
 #define PART3                   3
 #define PART4                   4
 
-#define MAX_ENCODING            ALPHAVMSi_ENCODING
+#define MAX_ENCODING            IA64VMSg_ENCODING
 #define MAX_COMPRESSION         GZIP_COMPRESSION
 #define MAX_CHECKSUM            MD5_CHECKSUM
 #define MAX_SETTINGS_LEN        200
@@ -758,13 +760,13 @@ char na[] = "n/a";
 char dots[] = "...";
 char *monos[] = { "n/a", "Unknown", "Increase", "Decrease", "False" };
 char *epochStyles[] = {
-  "standard", "alternate/1", "alternate/2", "alternate/3",
+  "standard", "alternate/1", "alternate/2", "ZISO8601",
   "Format/C_Fortran", "Format/custom", "ISO8601"
 };
 char *encodings[] = {
   NULL, "NETWORK", "SUN", "VAX", "DECSTATION", "SGi", "IBMPC", "IBMRS",
   "HOST", "PPC", NULL, "HP", "NeXT", "ALPHAOSF1", "ALPHAVMSd", "ALPHAVMSg",
-  "ALPHAVMSi", "PowerPC"
+  "ALPHAVMSi", "ARM_LITTLE", "ARM_BIG", "IA64VMSi", "IA64VMSd", "IA64VMSg"
 };
 char *majorities[] = { "input", "row", "column" };
 char *compressions[] = { "no", "rle", "huff", "ahuff", "gzip" };
@@ -808,6 +810,7 @@ int numGroups;
 char ***varsInGroup;
 struct ItemStruct ***varPtrsInGroup;
 int *numVarsInGroup;
+Logical ziso8601;
 #else
 extern int batchMode;
 extern Logical simpleMode;
@@ -864,6 +867,7 @@ extern int numGroups;
 extern char ***varsInGroup;
 extern struct ItemStruct ***varPtrsInGroup;
 extern int *numVarsInGroup;
+extern Logical ziso8601;
 #endif
 
 /******************************************************************************
