@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF				CDF Standard Interface (II) /FORTRAN.
 *
@@ -7,6 +12,11 @@
 *  Modification history:
 *
 *   V1.0  26-May-05, M Liu   	Original version (for CDF V3.1).
+*   V1.1  03-Jan-12, M Liu   	Added cdf_get_zvar_allrecords_varid,
+*                               cdf_get_zvar_rangerecords_varid,
+*                               cdf_get_var_allrecords_varname,
+*                               cdf_get_var_rangerecords_name functions, and
+*                               a set of similar functions for put operations.
 *
 ******************************************************************************/
 
@@ -920,7 +930,7 @@ Int32     *status;              /* Out: CDF status code. */
 {
 
   *status = (Int32) CDFlib (SELECT_, CDF_, Int32ToCDFid(*id),
-		                     zVAR_NUMBER_, (long) (*var_num -1),
+		                     zVAR_, (long) (*var_num -1),
 		            DELETE_, zVAR_,
 		            NULL_);
   return;
@@ -947,12 +957,66 @@ Int32     *status;              /* Out: CDF status code. */
   long startRec = (long) (*start_rec - 1);
   long endRec = (long) (*end_rec - 1);
   *status = (Int32) CDFlib (SELECT_, CDF_, Int32ToCDFid(*id),
-                                     zVAR_NUMBER_, (long) (*var_num -1),
+                                     zVAR_, (long) (*var_num -1),
                             DELETE_, zVAR_RECORDS_, startRec, endRec,
                             NULL_);
   return;
 }
 
+/******************************************************************************
+* CDF_delete_zvar_recs_renumber.
+******************************************************************************/
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_delete_zvar_recs_renumber__,
+               cdf_delete_zvar_recs_renumber_,
+               cdf_delete_zvar_recs_renumber,
+               CDF_DELETE_ZVAR_RECS_RENUMBER)
+(id, var_num, start_rec, end_rec, status)
+Int32     *id;                  /* In: CDF identifier. */
+Int32     *var_num;             /* In: zVariable number. */
+Int32     *start_rec;           /* In: zVariable's start record number. */
+Int32     *end_rec;             /* In: zVariable's end record number. */
+Int32     *status;              /* Out: CDF status code. */
+{
+  long startRec = (long) (*start_rec - 1);
+  long endRec = (long) (*end_rec - 1);
+  *status = (Int32) CDFlib (SELECT_, CDF_, Int32ToCDFid(*id),
+                                     zVAR_, (long) (*var_num -1),
+                            DELETE_, zVAR_RECORDS_RENUMBER_, startRec, endRec,
+                            NULL_);
+  return;
+}
+
+/******************************************************************************
+* CDF_insert_zvar_recs.
+******************************************************************************/
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_insert_zvar_recs__,
+               cdf_insert_zvar_recs_,
+               cdf_insert_zvar_recs,
+               CDF_INSERT_ZVAR_RECS)
+(id, var_num, start_rec, num_recs, buffer, status)
+Int32     *id;                  /* In: CDF identifier. */
+Int32     *var_num;             /* In: zVariable number. */
+Int32     *start_rec;           /* In: zVariable's start record number. */
+Int32     *num_recs;            /* In: Number of records to insert. */
+void      *buffer;              /* In: Buffer holding the inserted data. */
+Int32     *status;              /* Out: CDF status code. */
+{
+  long startRec = (long) (*start_rec - 1);
+  *status = (Int32) CDFinsertVarRecordsByVarID (Int32ToCDFid(*id), 1,
+                                                (long) (*var_num -1),
+                                                startRec, (long) *num_recs,
+                                                buffer);
+  return;
+}
+
 /******************************************************************************
 *  CDF_get_num_rvars.
 ******************************************************************************/
@@ -2056,18 +2120,338 @@ Int32     *rec_num;             /* In: CDF zVariable record number. */
 void      *buffer;              /* Out: zVariable's full record data. */
 Int32     *status;              /* Out: CDF status code. */
 {
-  long numVars, varNums[CDF_MAX_DIMS];
+  long numVars, varNums[1];
 
   numVars = 1;
   varNums[0] = (long) (*var_num - 1);
 
-  *status = (Int32) CDFlib (SELECT_, CDF_, id,
+  *status = (Int32) CDFlib (SELECT_, CDF_, Int32ToCDFid(*id),
                                      zVARs_RECNUMBER_, (long) (*rec_num - 1),
                             GET_, zVARs_RECDATA_, numVars, varNums, buffer,
                             NULL_);
   return;
 }
 
+/******************************************************************************
+*  CDF_get_zvar_allrecords_varid.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_get_zvar_allrecords_varid__,
+               cdf_get_zvar_allrecords_varid_,
+               cdf_get_zvar_allrecords_varid,
+               CDF_GET_ZVAR_ALLRECORDS_VARID)
+(id, var_num, buffer, status)
+Int32     *id;                  /* In: CDF identifier. */
+Int32     *var_num;             /* In: CDF zVariable number. */
+void      *buffer;              /* Out: zVariable's full record data. */
+Int32     *status;              /* Out: CDF status code. */
+{
+
+  *status = (Int32) CDFgetVarAllRecordsByVarID (Int32ToCDFid(*id), 1,
+                                                (long) (*var_num - 1),
+                                                buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_get_zvar_rangerecords_varid.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_get_zvar_rangerecords_varid__,
+               cdf_get_zvar_rangerecords_varid_,
+               cdf_get_zvar_rangerecords_varid,
+               CDF_GET_ZVAR_RANGERECORDS_VARID)
+(id, var_num, start_rec, stop_rec, buffer, status)
+Int32     *id;                  /* In: CDF identifier. */
+Int32     *var_num;             /* In: CDF zVariable number. */
+Int32     *start_rec;           /* In: Starting record number. */
+Int32     *stop_rec;            /* In: Stopping record number. */
+void      *buffer;              /* Out: zVariable's full record data. */
+Int32     *status;              /* Out: CDF status code. */
+{
+
+  *status = (Int32) CDFgetVarRangeRecordsByVarID (Int32ToCDFid(*id), 1,
+                                                  (long) (*var_num - 1),
+                                                  (long) (*start_rec - 1),
+                                                   (long) (*stop_rec - 1),
+                                                   buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_get_var_allrecords_varname.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_get_var_allrecords_varname__,
+               cdf_get_var_allrecords_varname_,
+               cdf_get_var_allrecords_varname,
+               CDF_GET_VAR_ALLRECORDS_VARNAME)
+(id, var_name, buffer, status Fif_GHOSTARG(len))
+Int32     *id;                  /* In: CDF identifier. */
+void      *var_name;            /* In: CDF zVariable name. */
+void      *buffer;              /* Out: zVariable's all records' data. */
+Int32     *status;              /* Out: CDF status code. */
+Fif_GHOSTDEF(len)               /* Invisible length of "var_name"
+                                   (generated by FORTRAN compiler). */
+{
+  struct STRINGstruct *ssh = NULL;
+  struct CDFstruct *CDF;
+  int LFS = FALSE;
+
+  CDF  = (struct CDFstruct *)Int32ToCDFid(*id);
+  if (isLFS(CDF)) LFS = TRUE;
+  *status = (Int32) CDFgetVarAllRecordsByVarName (Int32ToCDFid(*id),
+#if defined(Fif_DESCR)
+                                                  DESCRtoREFnul(var_name,
+                                                     (LFS ?
+                                                      CDF_VAR_NAME_LEN256 :
+                                                      CDF_VAR_NAME_LEN),
+                                                     &ssh),
+#endif
+#if defined(Fif_GHOSTLEN)
+                                                  NULterminate(var_name,
+                                                     Fif_GHOSTUSE(len),
+                                                     &ssh),
+#endif
+#if defined(Fif_NOLEN)
+                                                  FindEndNUL(var_name,
+                                                     (LFS ?
+                                                      CDF_VAR_NAME_LEN256 :
+                                                      CDF_VAR_NAME_LEN),
+                                                     &ssh),
+#endif
+                                                  buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_get_var_rangerecords_name.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_get_var_rangerecords_name__,
+               cdf_get_var_rangerecords_name_,
+               cdf_get_var_rangerecords_name,
+               CDF_GET_VAR_RANGERECORDS_NAME)
+(id, var_name, start_rec, stop_rec, buffer, status Fif_GHOSTARG(len))
+Int32     *id;                  /* In: CDF identifier. */
+void      *var_name;            /* In: CDF zVariable name. */
+Int32     *start_rec;           /* In: zVariable's starting record number. */
+Int32     *stop_rec;            /* In: zVariable's stopping record number. */
+void      *buffer;              /* Out: zVariable's records' data. */
+Int32     *status;              /* Out: CDF status code. */
+Fif_GHOSTDEF(len)               /* Invisible length of "var_name"
+                                   (generated by FORTRAN compiler). */
+{
+  struct STRINGstruct *ssh = NULL;
+  struct CDFstruct *CDF;
+  int LFS = FALSE;
+
+  CDF  = (struct CDFstruct *)Int32ToCDFid(*id);
+  if (isLFS(CDF)) LFS = TRUE;
+  *status = (Int32) CDFgetVarRangeRecordsByVarName (Int32ToCDFid(*id),
+#if defined(Fif_DESCR)
+                                                    DESCRtoREFnul(var_name,
+                                                       (LFS ?
+                                                        CDF_VAR_NAME_LEN256 :
+                                                        CDF_VAR_NAME_LEN),
+                                                       &ssh),
+#endif
+#if defined(Fif_GHOSTLEN)
+                                                    NULterminate(var_name,
+                                                       Fif_GHOSTUSE(len),
+                                                       &ssh),
+#endif
+#if defined(Fif_NOLEN)
+                                                    FindEndNUL(var_name,
+                                                       (LFS ?
+                                                        CDF_VAR_NAME_LEN256 :
+                                                        CDF_VAR_NAME_LEN),
+                                                       &ssh),
+#endif
+                                                    (long) (*start_rec - 1),
+                                                    (long) (*stop_rec - 1),
+                                                    buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_put_zvar_allrecords_varid.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_put_zvar_allrecords_varid__,
+               cdf_put_zvar_allrecords_varid_,
+               cdf_put_zvar_allrecords_varid,
+               CDF_PUT_ZVAR_ALLRECORDS_VARID)
+(id, var_num, num_recs, buffer, status)
+Int32     *id;                  /* In: CDF identifier. */
+Int32     *var_num;             /* In: CDF zVariable number. */
+Int32     *num_recs;            /* In: Number of zVariable records to write. */
+void      *buffer;              /* Out: zVariable's full record data. */
+Int32     *status;              /* Out: CDF status code. */
+{
+
+  *status = (Int32) CDFputVarAllRecordsByVarID (Int32ToCDFid(*id), 1,
+                                                (long) (*var_num - 1),
+                                                (long) *num_recs,
+                                                buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_put_zvar_rangerecords_varid.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_put_zvar_rangerecords_varid__,
+               cdf_put_zvar_rangerecords_varid_,
+               cdf_put_zvar_rangerecords_varid,
+               CDF_PUT_ZVAR_RANGERECORDS_VARID)
+(id, var_num, start_rec, stop_rec, buffer, status)
+Int32     *id;                  /* In: CDF identifier. */
+Int32     *var_num;             /* In: CDF zVariable number. */
+Int32     *start_rec;           /* In: Starting record number. */
+Int32     *stop_rec;            /* In: Stopping record number. */
+void      *buffer;              /* Out: zVariable's full record data. */
+Int32     *status;              /* Out: CDF status code. */
+{
+
+  *status = (Int32) CDFputVarRangeRecordsByVarID (Int32ToCDFid(*id), 1,
+                                                  (long) (*var_num - 1),
+                                                  (long) (*start_rec - 1),
+                                                  (long) (*stop_rec - 1),
+                                                  buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_put_var_allrecords_varname.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_put_var_allrecords_varname__,
+               cdf_put_var_allrecords_varname_,
+               cdf_put_var_allrecords_varname,
+               CDF_PUT_VAR_ALLRECORDS_VARNAME)
+(id, var_name, num_recs, buffer, status Fif_GHOSTARG(len))
+Int32     *id;                  /* In: CDF identifier. */
+void      *var_name;            /* In: CDF zVariable name. */
+Int32     *num_recs;            /* In: Number of records to write. */
+void      *buffer;              /* Out: zVariable's all records' data. */
+Int32     *status;              /* Out: CDF status code. */
+Fif_GHOSTDEF(len)               /* Invisible length of "var_name"
+                                   (generated by FORTRAN compiler). */
+{
+  struct STRINGstruct *ssh = NULL;
+  struct CDFstruct *CDF;
+  int LFS = FALSE;
+
+  CDF  = (struct CDFstruct *)Int32ToCDFid(*id);
+  if (isLFS(CDF)) LFS = TRUE;
+  *status = (Int32) CDFputVarAllRecordsByVarName (Int32ToCDFid(*id),
+#if defined(Fif_DESCR)
+                                                  DESCRtoREFnul(var_name,
+                                                     (LFS ?
+                                                      CDF_VAR_NAME_LEN256 :
+                                                      CDF_VAR_NAME_LEN),
+                                                     &ssh),
+#endif
+#if defined(Fif_GHOSTLEN)
+                                                  NULterminate(var_name,
+                                                     Fif_GHOSTUSE(len),
+                                                     &ssh),
+#endif
+#if defined(Fif_NOLEN)
+                                                  FindEndNUL(var_name,
+                                                     (LFS ?
+                                                      CDF_VAR_NAME_LEN256 :
+                                                      CDF_VAR_NAME_LEN),
+                                                     &ssh),
+#endif
+                                                  (long) *num_recs,
+                                                  buffer);
+  return;
+}
+
+/******************************************************************************
+*  CDF_put_var_rangerecords_name.
+******************************************************************************/
+
+VISIBLE_PREFIX
+Fif_PREFIXa
+void
+Fif_PREFIXb
+Fif_ENTRYPOINT(cdf_put_var_rangerecords_name__,
+               cdf_put_var_rangerecords_name_,
+               cdf_put_var_rangerecords_name,
+               CDF_PUT_VAR_RANGERECORDS_NAME)
+(id, var_name, start_rec, stop_rec, buffer, status Fif_GHOSTARG(len))
+Int32     *id;                  /* In: CDF identifier. */
+void      *var_name;            /* In: CDF zVariable name. */
+Int32     *start_rec;           /* In: zVariable's starting record number. */
+Int32     *stop_rec;            /* In: zVariable's stopping record number. */
+void      *buffer;              /* Out: zVariable's records' data. */
+Int32     *status;              /* Out: CDF status code. */
+Fif_GHOSTDEF(len)               /* Invisible length of "var_name"
+                                   (generated by FORTRAN compiler). */
+{
+  struct STRINGstruct *ssh = NULL;
+  struct CDFstruct *CDF;
+  int LFS = FALSE;
+  
+  CDF  = (struct CDFstruct *)Int32ToCDFid(*id);
+  if (isLFS(CDF)) LFS = TRUE;
+  *status = (Int32) CDFputVarRangeRecordsByVarName (Int32ToCDFid(*id),
+#if defined(Fif_DESCR)
+                                                    DESCRtoREFnul(var_name,
+                                                       (LFS ?
+                                                        CDF_VAR_NAME_LEN256 :
+                                                        CDF_VAR_NAME_LEN),
+                                                       &ssh),
+#endif
+#if defined(Fif_GHOSTLEN)
+                                                    NULterminate(var_name,
+                                                       Fif_GHOSTUSE(len),
+                                                       &ssh),
+#endif
+#if defined(Fif_NOLEN)
+                                                    FindEndNUL(var_name,
+                                                       (LFS ?
+                                                        CDF_VAR_NAME_LEN256 :
+                                                        CDF_VAR_NAME_LEN),
+                                                       &ssh),
+#endif
+                                                    (long) (*start_rec - 1),
+                                                    (long) (*stop_rec - 1),
+                                                    buffer);
+  return;
+}
+
 /******************************************************************************
 *  CDF_get_checksum.
 ******************************************************************************/
