@@ -69,7 +69,7 @@ OFF_T b;
 * it, the data will be properly preserved.
 ******************************************************************************/
 
-STATICforIDL CDFstatus UpdateDotCDF64 (CDF)
+VISIBLE_PREFIX CDFstatus UpdateDotCDF64 (CDF)
 struct CDFstruct *CDF; 
 {
   CDFstatus pStatus = CDF_OK; int varN; struct VarStruct *Var; Logical zVar;
@@ -392,6 +392,48 @@ CDFstatus *pStatus;     /* Returned status. */
 #endif
 }
 
+/******************************************************************************
+* UpdateTT2000header.
+* The leap second last updated header is to be updated from the leap second
+* table if it is an earlier day (not zero).
+******************************************************************************/
+
+STATICforIDL Logical UpdateTT2000header (CDF, pStatus)
+struct CDFstruct *CDF;
+CDFstatus *pStatus;
+{
+   long year, month, day;
+   Int32 tLastUpdated, tLastUpdated2;
+   /***************************************************************************
+   * Check if updated already.
+   ***************************************************************************/
+   if (CDF->tt2000Updated == 1) return TRUE;
+   if (!sX(ReadGDR64(CDF->fp,CDF->GDRoffset64,
+                     GDR_LEAPSECONDLASTUPDATED,&tLastUpdated,
+                     GDR_NULL),pStatus)) {
+     AbortAccess64 (CDF, noUPDATE, noDELETE);
+     return FALSE;
+   }
+   if (tLastUpdated != 0) {
+#if defined(vms)
+     CDFgetLastDateinLeapSecondsTBL (&year, &month, &day);
+#else
+     CDFgetLastDateinLeapSecondsTable (&year, &month, &day);
+#endif
+     tLastUpdated2 = 10000*(Int32)year + 100*(Int32)month + (Int32)day;
+     if (tLastUpdated2 > tLastUpdated) { 
+       if (!sX(WriteGDR64(CDF->fp,CDF->GDRoffset64,
+                          GDR_LEAPSECONDLASTUPDATED,&tLastUpdated2,
+                          GDR_NULL),pStatus)) {
+         AbortAccess64 (CDF, noUPDATE, noDELETE);
+         return FALSE;
+       }
+     }
+   }
+   CDF->tt2000Updated = 1;
+   return TRUE;
+}
+
 /******************************************************************************
 * WriteBuffer64.
 *    Write occurs at current offset (assumed to have been set before this
@@ -1163,7 +1205,7 @@ struct CDFstruct *CDF;
   if (CDF->dotFp == NULL) {
     BuildFilePath (CDFt, CDF->CDFname, CDF->no_append, CDF->upper_case_ext,
                    CDF->version_numbers, INT32_ZERO, tmpFile);
-    if (!DeleteFile(tmpFile)) sX (CDF_DELETE_ERROR, &pStatus);
+    if (!CDFdeleteFile(tmpFile)) sX (CDF_DELETE_ERROR, &pStatus);
   }
   else {
     if (!DELETEv64(CDF->dotFp,NULL)) sX (CDF_DELETE_ERROR, &pStatus);
@@ -1186,7 +1228,7 @@ struct CDFstruct *CDF;
        }
        BuildFilePath (Vt, CDF->CDFname, CDF->no_append, CDF->upper_case_ext,
                       CDF->version_numbers, varN, tmpFile);
-       if (!DeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
+       if (!CDFdeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
     }
     for (varN = 0; varN < CDF->NzVars; varN++) {
        struct VarStruct *Var = CDF->zVars[varN];
@@ -1199,7 +1241,7 @@ struct CDFstruct *CDF;
        }
        BuildFilePath (Zt, CDF->CDFname, CDF->no_append, CDF->upper_case_ext,
                       CDF->version_numbers, varN, tmpFile);
-       if (!DeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
+       if (!CDFdeleteFile(tmpFile)) sX (VAR_DELETE_ERROR, &pStatus);
     }
   }
   return pStatus;
@@ -1412,3 +1454,94 @@ void *buffer;
   return CDF_OK;
 }
 
+/******************************************************************************
+* SetLeapSecondLastUpdate.
+* Set the last updated date for CDF that has TT2000 variable.
+******************************************************************************/
+
+STATICforIDL CDFstatus SetLeapSecondLastUpdate (CDF, Var, zVar)
+struct CDFstruct *CDF;
+struct VarStruct *Var;      /* the variable */
+Logical zVar;
+{
+  Int32 dataType, LeapSecondLastUpdated, version, release, increment;
+  long year, month, day;
+   CDFstatus pStatus = CDF_OK;
+  if (CDF->leapSecondUpdated == 1) return CDF_OK;
+  if (!sX(ReadVDR64(CDF,CDF->fp,Var->VDRoffset64,zVar,
+                    VDR_DATATYPE,&dataType,
+                    VDR_NULL),&pStatus)) {
+    AbortAccess64 (CDF, noUPDATE, noDELETE);
+    return pStatus;
+  }
+  if (dataType != CDF_TIME_TT2000) return CDF_OK;
+  if (!sX(ReadCDR64(CDF->fp,V3_CDR_OFFSET64,
+                    CDR_VERSION,&version,
+                    CDR_RELEASE,&release,
+                    CDR_INCREMENT,&increment,
+                    CDR_NULL),&pStatus)) {
+    AbortAccess64 (CDF, noUPDATE, noDELETE);
+    return pStatus;
+  }
+  /*********************************************************************
+  * Update version/release/increment and copywright only if a newer
+  * library version is modifying the CDF.
+  *********************************************************************/
+  if (!PriorTo("3.6.0",version,release,increment)) {
+#if defined(vms)
+    CDFgetLastDateinLeapSecondsTBL (&year, &month, &day);
+#else
+    CDFgetLastDateinLeapSecondsTable (&year, &month, &day);
+#endif
+    LeapSecondLastUpdated = (Int32) (10000*year + 100*month + day);
+    if (!sX(WriteGDR64(CDF->fp,CDF->GDRoffset64,
+                       GDR_LEAPSECONDLASTUPDATED,&LeapSecondLastUpdated,
+                       GDR_NULL),&pStatus)) {
+      AbortAccess64 (CDF, noUPDATE, CDFDELETE);
+      cdf_FreeMemory (CDF, NULL);
+      return pStatus;
+    }
+    CDF->leapSecondUpdated = 1;
+  }
+  return CDF_OK;
+}
+
+/******************************************************************************
+* StringBreakIntoStrings.
+* Break a single string into mutliple strings. Each string is separated by the
+* defined STRINGDELIMITER in the original string.
+******************************************************************************/
+
+VISIBLE_PREFIX void StringBreakIntoStrings (string, numElems, numStrs, strings)
+char *string;
+long numElems;
+long numStrs;
+char ***strings;
+{
+  if (numStrs < 2) {
+    *strings = (char **) malloc (sizeof(char *));
+    (*strings)[0] = malloc ((size_t)numElems+1);
+    memcpy((*strings)[0], string, (size_t)numElems+1);
+  } else {
+    int ix; size_t length; char *dstr, *cstr;
+    cstr = string;
+    *strings = (char **) malloc (sizeof(char *) * numStrs);
+    for (ix = 0; ix < numStrs; ++ix) {
+      dstr = strstr(cstr, STRINGDELIMITER);
+      if (dstr != NULL) {
+        length = dstr - cstr;
+        (*strings)[ix] = (char *) malloc (sizeof(char)*length +1);
+        memcpy ((*strings)[ix], cstr, length);
+        *((*strings)[ix]+(int)length) = '\0';
+        cstr = dstr + (int) strlen(STRINGDELIMITER);
+      } else {
+        length = strlen(cstr);
+        (*strings)[ix] = (char *) malloc (sizeof(char)*length +1);
+        memcpy ((*strings)[ix], cstr, length);
+        *((*strings)[ix]+(int)length) = '\0';
+      }
+    }
+  }
+}
+
+
