@@ -482,6 +482,9 @@ switch (Va->item) {
     Logical zOp = (Va->item == zVAR_BLOCKINGFACTOR_);
     struct CDFstruct *CDF; struct VarStruct *Var;
     Int32 blocking = (Int32) va_arg (Va->ap, long), flags;
+    Int32 recSize, numDims, dimSizes[CDF_MAX_DIMS], dimVarys[CDF_MAX_DIMS], 
+          numElems, dataType;
+    int i;
     SelectCDF (Cur->cdf, CDF)
     if (BADzOP(CDF,!zOp)) return ILLEGAL_IN_zMODE;
     if (!CURRENTvarSELECTED(CDF,zOp)) return NO_VAR_SELECTED;
@@ -518,17 +521,48 @@ switch (Va->item) {
     if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
     /**************************************************************************
     * Update VDR with new blocking factor.
+    * Make sure that the blocking factor is at least the size of default for
+    * any compressed variable. Otherwise, don't change/set it.
     **************************************************************************/
     if (!sX(ReadVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
 		      VDR_FLAGS,&flags,
+		      VDR_DIMVARYS,dimVarys,
+		      VDR_DATATYPE,&dataType,
+		      VDR_NUMELEMS,&numElems,
 		      VDR_NULL),&pStatus)) {
       AbortAccess64 (CDF, UPDATE, noDELETE);
       return pStatus;
     }
+    if (zOp) {
+      if (!sX(ReadVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
+			VDR_zNUMDIMS,&numDims,
+			VDR_zDIMSIZES,dimSizes,
+			VDR_NULL),&pStatus)) {
+        AbortAccess64 (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+    } else {
+      if (!sX(ReadGDR64(CDF->fp,CDF->GDRoffset64,
+			GDR_rNUMDIMS,&numDims,
+			GDR_rDIMSIZES,dimSizes,
+			GDR_NULL),&pStatus)) {
+        AbortAccess64 (CDF, UPDATE, noDELETE);
+        return pStatus;
+      }
+    }
     if (!RECvaryBITset(flags) && blocking > 1) {
       blocking = 1;
       sX (FORCED_PARAMETER, &pStatus);
     }
+    if (VARcompressionBITset(flags) && RECvaryBITset(flags) && (blocking != 0)) {
+      recSize = numElems;
+      recSize *= (Int32) CDFelemSize(dataType);
+      for (i = 0; i < (int) numDims; ++i) if (dimVarys[i]) recSize *= dimSizes[i];
+      if (blocking < (Int32) (MIN_BLOCKING_BYTES_compressed/recSize)) {
+	sX (BLOCKINGFACTOR_TOO_SMALL2, &pStatus);
+/*	return pStatus; */
+      }
+    }
     if (!sX(WriteVDR64(CDF,CDF->fp,Var->VDRoffset64,Var->zVar,
 		       VDR_BLOCKING,&blocking,
 		       VDR_NULL),&pStatus)) {
@@ -568,6 +602,9 @@ switch (Va->item) {
     * Get write access.
     **************************************************************************/
     if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
+    if (Var->dataType == CDF_TIME_TT2000) {
+      if (!UpdateTT2000header(CDF,&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * Read the variable's VDR.
     **************************************************************************/
@@ -699,6 +736,9 @@ switch (Va->item) {
     * Get write access.
     **************************************************************************/
     if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
+    if (Var->dataType == CDF_TIME_TT2000) {
+      if (!UpdateTT2000header(CDF,&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * If a multi-file CDF, open variable file (if closed).
     **************************************************************************/
@@ -726,6 +766,10 @@ switch (Va->item) {
     * Update last access time.
     **************************************************************************/
     Var->accessed_at = CDF->pseudo_clock++;
+    if (!sX(SetLeapSecondLastUpdate (CDF, Var, zOp), &pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
     break;
   }
 
@@ -779,6 +823,9 @@ switch (Va->item) {
     * Get write access.
     **************************************************************************/
     if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
+    if (Var->dataType == CDF_TIME_TT2000) {
+      if (!UpdateTT2000header(CDF,&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * If a multi-file CDF, open variable file (if closed).
     **************************************************************************/
@@ -799,6 +846,10 @@ switch (Va->item) {
     * Update last access time.
     **************************************************************************/
     Var->accessed_at = Cur->cdf->pseudo_clock++;
+    if (!sX(SetLeapSecondLastUpdate (CDF, Var, zOp), &pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
     break;
   }
 
@@ -839,6 +890,9 @@ switch (Va->item) {
     * Get write access.
     **************************************************************************/
     if (!WriteAccess64(CDF,FALSE,&pStatus)) return pStatus;
+    if (Var->dataType == CDF_TIME_TT2000) {
+      if (!UpdateTT2000header(CDF,&pStatus)) return pStatus;
+    }
     /**************************************************************************
     * If a multi-file CDF, open variable file (if closed).
     **************************************************************************/
@@ -863,6 +917,10 @@ switch (Va->item) {
     **************************************************************************/
     Var->seqValueOffset64++;
     Var->accessed_at = Cur->cdf->pseudo_clock++;
+    if (!sX(SetLeapSecondLastUpdate (CDF, Var, zOp), &pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
     break;
   }
 
@@ -953,7 +1011,10 @@ switch (Va->item) {
        tBuffer += (size_t) Var->NphyRecBytes;
        Var->accessed_at = CDF->pseudo_clock++;
     }
-
+    if (!sX(SetLeapSecondLastUpdate (CDF, Var, zOp), &pStatus)) {
+      AbortAccess64 (CDF, UPDATE, noDELETE);
+      return pStatus;
+    }
     break;
   }
   /****************************************************************************
@@ -967,7 +1028,9 @@ switch (Va->item) {
   case ATTR_SCOPE_:
   case gENTRY_DATA_:
   case rENTRY_DATA_:
+  case rENTRY_STRINGSDATA_:
   case zENTRY_DATA_:
+  case zENTRY_STRINGSDATA_:
   case gENTRY_DATASPEC_:
   case rENTRY_DATASPEC_:
   case zENTRY_DATASPEC_:
@@ -978,6 +1041,7 @@ switch (Va->item) {
   case rVAR_SPARSEARRAYS_:
   case zVAR_SPARSEARRAYS_:
   case CDF_CHECKSUM_:
+  case CDF_LEAPSECONDLASTUPDATED_:
     return CDFput2_64 (Va, Cur);
   /****************************************************************************
   * Unknown item, must be the next function.
