@@ -1,4 +1,9 @@
 /******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+/******************************************************************************
 *
 *  NSSDC/CDF                                        Read from internal record.
 *
@@ -237,7 +242,7 @@ va_dcl
   ***************************************************************************/
   if (read_only_mode == READONLYon && fp->GDR == NULL)
   {
-      fp->GDR = cdf_AllocateMemory(sizeof(struct GDRstruct), NULL);
+      fp->GDR = cdf_AllocateMemory((size_t)sizeof(struct GDRstruct), NULL);
       if (fp->GDR == NULL) return CRE;
       if (!SEEKv(fp,(long)offset,vSEEK_SET)) return CRE;
       if (!Read32(fp,&(fp->GDR->RecordSize))) return CRE;
@@ -741,7 +746,7 @@ va_dcl
 #endif
 {
   long read_only_mode;
-  struct AEDRstructExt *CurAEDR;
+  struct AEDRstructExt *CurAEDR = NULL;
   va_list ap; CDFstatus pStatus = CDF_OK;
 #if defined(STDARG)
   va_start (ap, offset);
@@ -824,15 +829,20 @@ va_dcl
              if (!Read32(fp,&(AEDR->DataType))) return CRE;
              if (!Read32(fp,&(AEDR->Num))) return CRE;
              if (!Read32(fp,&(AEDR->NumElems))) return CRE;
-             if (!Read32(fp,&(AEDR->rfuA))) return CRE;
+             if (!Read32(fp,&(AEDR->NumStrings))) return CRE;
              if (!Read32(fp,&(AEDR->rfuB))) return CRE;
              if (!Read32(fp,&(AEDR->rfuC))) return CRE;
              if (!Read32(fp,&(AEDR->rfuD))) return CRE;
              if (!Read32(fp,&(AEDR->rfuE))) return CRE;
              if (value != NULL) {
-                 nBytes = (size_t) (CDFelemSize(AEDR->DataType) *
-                          AEDR->NumElems);
-                 if (!READv(value,nBytes,1,fp)) return CRE;
+               nBytes = (size_t) (CDFelemSize(AEDR->DataType) *
+                                  AEDR->NumElems);
+               if (!READv(value,nBytes,1,fp)) return CRE;
+               if (STRINGdataType(AEDR->DataType)) {
+                 FillNulsToString((char *)value, (int)AEDR->NumElems,
+                                  (int)AEDR->NumElems);
+               }
+
              };
          }
          else
@@ -844,7 +854,7 @@ va_dcl
              AEDR->DataType = CurAEDR->AEDR.DataType;
              AEDR->Num = CurAEDR->AEDR.Num;
              AEDR->NumElems = CurAEDR->AEDR.NumElems;
-             AEDR->rfuA = CurAEDR->AEDR.rfuA;
+             AEDR->NumStrings = CurAEDR->AEDR.NumStrings;
              AEDR->rfuB = CurAEDR->AEDR.rfuB;
              AEDR->rfuC = CurAEDR->AEDR.rfuC;
              AEDR->rfuD = CurAEDR->AEDR.rfuD;
@@ -868,6 +878,9 @@ va_dcl
 	     if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
 	     nBytes = (size_t) (CDFelemSize(dataType) * numElems);
 	     if (!READv(value,nBytes,1,fp)) return CRE;
+             if (STRINGdataType(dataType)) {
+               FillNulsToString((char *)value, (int)numElems, (int)numElems);
+             }
          }
          else
          {
@@ -955,6 +968,17 @@ va_dcl
                };
                break;
            }
+           case AEDR_NUMSTRINGS: {
+               if (read_only_mode == READONLYoff)
+               {
+                   tOffset += (OFF_T) AEDR_NUMSTRINGS_OFFSET; break;
+               }
+               else
+               {
+                   *buffer = CurAEDR->AEDR.NumStrings;
+               }
+               break;
+           }
 	   default: return CDF_INTERNAL_ERROR;
 	 }
          if (read_only_mode == READONLYoff)
@@ -1051,6 +1075,10 @@ va_dcl
 	 if (PADvalueBITset(VDR->Flags) && padValue != NULL) {
 	   size_t nBytes = (size_t) (CDFelemSize(VDR->DataType)*VDR->NumElems);
 	   if (!READv(padValue,nBytes,1,fp)) return CRE;
+           if (STRINGdataType(VDR->DataType)) {
+             FillNulsToString((char *)padValue, (int)VDR->NumElems,
+                              (int)VDR->NumElems);
+           }
 	 }
 	 break;
        }
@@ -1146,6 +1174,10 @@ va_dcl
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CRE;
 	 nBytes = (size_t) (CDFelemSize(dataType) * numElems);
 	 if (!READv(padValue,nBytes,1,fp)) return CRE;
+         if (STRINGdataType(dataType)) {
+           FillNulsToString((char *)padValue, (int)numElems,
+                            (int)numElems);
+         }
 	 break;
        }
        default: {
@@ -1612,12 +1644,12 @@ STATICforIDL CDFstatus ReadADRList (vFILE *fp)
   * Allocate memory for the list of ADRs
   ***************************************************************************/
   if (fp->GDR->NumAttr > 0)
-  fp->ADRList = cdf_AllocateMemory(fp->GDR->NumAttr * 
+  fp->ADRList = cdf_AllocateMemory((size_t)fp->GDR->NumAttr * 
                                    sizeof(struct ADRstruct**), NULL);
   if (fp->GDR->NumAttr > 0 && fp->ADRList == NULL) return BAD_MALLOC;
   for (i = 0; i < fp->GDR->NumAttr; i++)
   {
-      fp->ADRList[i] = cdf_AllocateMemory(sizeof(struct ADRstruct), NULL);
+      fp->ADRList[i] = cdf_AllocateMemory((size_t)sizeof(struct ADRstruct), NULL);
       if (fp->ADRList[i] == NULL) return BAD_MALLOC;
   };
   /***************************************************************************
@@ -1683,7 +1715,7 @@ STATICforIDL CDFstatus ReadAEDRList (vFILE *fp,
   * Allocate memory for the list and initialize each entry to NULL.
   **************************************************************************/
   if (MaxEntry >= 0) {
-      *AEDRList = cdf_AllocateMemory((MaxEntry + 1) * 
+      *AEDRList = cdf_AllocateMemory((size_t)(MaxEntry + 1) * 
                                      sizeof(struct AEDRstructExt**), NULL);
       if (*AEDRList == NULL) return BAD_MALLOC;
   }
@@ -1704,7 +1736,7 @@ STATICforIDL CDFstatus ReadAEDRList (vFILE *fp,
   ***************************************************************************/
   while (NxtAEDR != 0 && iEntries != NumEntries)
   {
-      TempAEDR = cdf_AllocateMemory(sizeof(struct AEDRstructExt), NULL);
+      TempAEDR = cdf_AllocateMemory((size_t)sizeof(struct AEDRstructExt), NULL);
       if (TempAEDR == NULL) return BAD_MALLOC;
       if (!SEEKv(fp,NxtAEDR,vSEEK_SET)) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.RecordSize))) return CRE;
@@ -1715,7 +1747,7 @@ STATICforIDL CDFstatus ReadAEDRList (vFILE *fp,
       if (!Read32(fp,&(TempAEDR->AEDR.DataType))) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.Num))) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.NumElems))) return CRE;
-      if (!Read32(fp,&(TempAEDR->AEDR.rfuA))) return CRE;
+      if (!Read32(fp,&(TempAEDR->AEDR.NumStrings))) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.rfuB))) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.rfuC))) return CRE;
       if (!Read32(fp,&(TempAEDR->AEDR.rfuD))) return CRE;
@@ -1723,9 +1755,14 @@ STATICforIDL CDFstatus ReadAEDRList (vFILE *fp,
       TempAEDR->ValueSize = (CDFelemSize(TempAEDR->AEDR.DataType) *
                              TempAEDR->AEDR.NumElems);
       if (TempAEDR->ValueSize < 1) return CV2C;
-      TempAEDR->Value = cdf_AllocateMemory(TempAEDR->ValueSize, NULL);
+      TempAEDR->Value = cdf_AllocateMemory((size_t)TempAEDR->ValueSize, NULL);
       if (TempAEDR->Value == NULL) return BAD_MALLOC;
       if (!READv(TempAEDR->Value,TempAEDR->ValueSize,1,fp)) return CRE;
+      if (STRINGdataType(TempAEDR->AEDR.DataType)) {
+        FillNulsToString((char *)(TempAEDR->Value),
+                         (int)TempAEDR->AEDR.NumElems,
+                         (int)TempAEDR->AEDR.NumElems);
+      }
       ++iEntries;
       if (TempAEDR->AEDR.Num < 0 || TempAEDR->AEDR.Num > MaxEntry)
         return CV2C;
