@@ -389,14 +389,31 @@ va_dcl
 	 if (!Write32(fp,&(AEDR->DataType))) return CWE;
 	 if (!Write32(fp,&(AEDR->Num))) return CWE;
 	 if (!Write32(fp,&(AEDR->NumElems))) return CWE;
-	 if (!Write32(fp,&(AEDR->rfuA))) return CWE;
+	 if (!Write32(fp,&(AEDR->NumStrings))) return CWE;
 	 if (!Write32(fp,&(AEDR->rfuB))) return CWE;
 	 if (!Write32(fp,&(AEDR->rfuC))) return CWE;
 	 if (!Write32(fp,&(AEDR->rfuD))) return CWE;
 	 if (!Write32(fp,&(AEDR->rfuE))) return CWE;
-	 if (value != NULL) {
-	   if (!sX(WriteBuffer(CDF,fp,AEDR->DataType,
-			       AEDR->NumElems,value),&pStatus)) return pStatus;
+         if (value != NULL) {
+	   if (STRINGdataType(AEDR->DataType)) {
+	     size_t len; char *newValue = NULL;
+	     len = strlen ((char *)value);
+	     if ((int)len < (int) AEDR->NumElems) {
+	       int ix;
+	       newValue = (char *) cdf_AllocateMemory ((size_t) AEDR->NumElems,
+						       NULL);
+	       memcpy (newValue, (char *)value, len);
+	       for (ix = (int) len; ix < (int) AEDR->NumElems; ++ix)
+	         *(newValue+ix) = (char) '\0';
+	     }
+	     if (!sX(WriteBuffer(CDF,fp,AEDR->DataType,AEDR->NumElems,
+				 (newValue==NULL?value:(void *)newValue)),
+		     &pStatus))
+	       return pStatus;
+	     if (newValue != NULL) cdf_FreeMemory (newValue, NULL);
+	   } else
+	     if (!sX(WriteBuffer(CDF,fp,AEDR->DataType,
+				 AEDR->NumElems,value),&pStatus)) return pStatus;
 	 }
 	 break;
        }
@@ -409,8 +426,24 @@ va_dcl
 			  AEDR_NULL),&pStatus)) return pStatus;
 	 tOffset = offset + AEDR_VALUE_OFFSET;
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
-	 if (!sX(WriteBuffer(CDF,fp,dataType,
-			     numElems,value),&pStatus)) return pStatus;
+	 if (STRINGdataType(dataType)) {
+	   size_t len; char *newValue = NULL;
+	   len = strlen ((char *)value);
+	   if ((int)len < (int) numElems) {
+	     int ix;
+	     newValue = (char *) cdf_AllocateMemory ((size_t) numElems, NULL);
+	     memcpy (newValue, (char *)value, len);
+	     for (ix = (int) len; ix < (int) numElems; ++ix)
+	        *(newValue+ix) = (char) '\0';
+	   }
+	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,
+			       (newValue==NULL?value:(void *)newValue)),
+		   &pStatus))
+	     return pStatus;
+	   if (newValue != NULL) cdf_FreeMemory (newValue, NULL);
+	 } else
+	   if (!sX(WriteBuffer(CDF,fp,dataType,
+			       numElems,value),&pStatus)) return pStatus;
 	 break;
        }
        default: {
@@ -423,6 +456,7 @@ va_dcl
 	   case AEDR_DATATYPE: tOffset += AEDR_DATATYPE_OFFSET; break;
 	   case AEDR_NUM: tOffset += AEDR_NUM_OFFSET; break;
 	   case AEDR_NUMELEMS: tOffset += AEDR_NUMELEMS_OFFSET; break;
+	   case AEDR_NUMSTRINGS: tOffset += AEDR_NUMSTRINGS_OFFSET; break;
 	   default: return CDF_INTERNAL_ERROR;
 	 }
 	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
@@ -505,9 +539,26 @@ va_dcl
 	 }
 	 if (!Write32s(fp,VDR->DimVarys,(int)nDims)) return CWE;
 	 if (PADvalueBITset(VDR->Flags) && padValue != NULL) {
-	   if (!sX(WriteBuffer(CDF,fp,VDR->DataType,
-			       VDR->NumElems,
-			       padValue),&pStatus)) return pStatus;
+	   if (STRINGdataType(VDR->DataType)) {
+	     size_t len; char *newPad = NULL;
+	     len = strlen ((char *)padValue);
+	     if ((int)len < (int) VDR->NumElems) {
+	       int ix;
+	       newPad = (char *) cdf_AllocateMemory ((size_t) VDR->NumElems,
+						     NULL);
+	       memcpy (newPad, (char *)padValue, len);
+	       for (ix = (int) len; ix < (int) VDR->NumElems; ++ix)
+		 *(newPad+ix) = (char) '\0';
+	     }
+	     if (!sX(WriteBuffer(CDF,fp,VDR->DataType,VDR->NumElems,
+				 (newPad==NULL?padValue:(void *)newPad)),
+		     &pStatus))
+	       return pStatus;
+	     if (newPad != NULL) cdf_FreeMemory (newPad, NULL);
+	   } else
+	     if (!sX(WriteBuffer(CDF,fp,VDR->DataType,
+			         VDR->NumElems,
+			         padValue),&pStatus)) return pStatus;
 	 }
 	 break;
        }
@@ -587,29 +638,25 @@ va_dcl
 			   GDR_NULL),&pStatus)) return pStatus;
 	   tOffset = offset + rVDR_PADVALUE_OFFSETb + (rNumDims*sizeof(Int32));
 	 }
+	 if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
 	 if (STRINGdataType(dataType)) {
-	   size_t len;
+	   size_t len; char *newPad = NULL;
 	   len = strlen ((char *)padValue);
-	   if ((int)len >= numElems) {
-	     if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
-	     if (!sX(WriteBuffer(CDF,fp,dataType,
-				 numElems,padValue),&pStatus)) return pStatus;
-	   } else {
-	     int ix; char *newPad;
+	   if ((int)len < (int) numElems) {
+	     int ix;
 	     newPad = (char *) cdf_AllocateMemory ((size_t) numElems, NULL);
-	     strncpy (newPad, (char *)padValue, len);
-	     for (ix = (int) len; ix < numElems; ++ix)
-		*(newPad+ix) = (char) ' ';
-	     if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
-	     if (!sX(WriteBuffer(CDF,fp,dataType,
-	                         numElems,newPad),&pStatus)) return pStatus;
-	     cdf_FreeMemory (newPad, NULL);
+	     memcpy (newPad, (char *)padValue, len);
+	     for (ix = (int) len; ix < (int) numElems; ++ix)
+	        *(newPad+ix) = (char) '\0';
 	   }
-	 } else {
-	   if (!SEEKv(fp,tOffset,vSEEK_SET)) return CWE;
+	   if (!sX(WriteBuffer(CDF,fp,dataType,numElems,
+			       (newPad==NULL?padValue:(void *)newPad)),
+		   &pStatus))
+	     return pStatus;
+	   if (newPad != NULL) cdf_FreeMemory (newPad, NULL);
+	 } else
 	   if (!sX(WriteBuffer(CDF,fp,dataType,
-	                       numElems,padValue),&pStatus)) return pStatus;
-	 }
+			       numElems,padValue),&pStatus)) return pStatus;
 	 break;
        }
        default: {
