@@ -46,6 +46,9 @@
 *                               sure the actual read/write bytes is within
 *                               the buffer size.
 *   V5.4  12-Aug-10, M Liu      Fill 0s to each new cache in AllocateBuffer.
+*   V5.5  10-Jan-17, M Liu      Modified to use mkstemp for creating temporary
+*                               file for Unix-based systems. Also, change the
+*                               directory to which the file will be created.
 *
 ******************************************************************************/
 
@@ -55,7 +58,9 @@
 
 #include "cdflib.h"
 #include "cdflib64.h"
-#include "errno.h"
+#if defined(win32)
+#  include "errno.h"
+#endif
 
 /******************************************************************************
 * Local macros/typedef's.
@@ -82,16 +87,22 @@ BOO(useVmem,LoadVMemory((MemHandle)cache->ptr,TRUE),cache->ptr)
 #define CACHEbufferWRITEto(cache) cache->ptr
 #endif
 
+#define TmpFileTemplate "mycdftmp.XXXXXX"
+
+int pid = -1;
+
 /******************************************************************************
 * Local function prototypes.
 ******************************************************************************/
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-static int OpenFile PROTOARGs((char *file_spec, char *a_mode));
+static int CDFOpenFile PROTOARGs((char *file_spec, char *a_mode));
 #else
-static FILE *OpenFile PROTOARGs((char *file_spec, char *a_mode));
+static FILE *CDFOpenFile PROTOARGs((char *file_spec, char *a_mode));
 #endif
+
 static Logical FreeCache PROTOARGs((vCACHE *firstCache));
 static vCACHE *FindCache PROTOARGs((vFILE *vFp, long blockN));
+static int CountCaches PROTOARGs((vFILE *vFp));
 static Logical vRead PROTOARGs((
   OFF_T offset, void *buffer, size_t nBytes, vFILE *vFp
 ));
@@ -106,17 +117,18 @@ static Logical WriteBlockFromCache PROTOARGs((
 static Logical WriteBlockFromBuffer PROTOARGs((
   vFILE *vFp, long blockN, void *buffer, size_t Nbytes
 ));
+
 #if EXTEND_FILE
 static Logical ExtendFile PROTOARGs((vFILE *vFp, long toBlockN));
 #endif
 
 /******************************************************************************
-* OpenFile.
+* CDFOpenFile.
 ******************************************************************************/
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-static int OpenFile (file_spec, a_mode)
+static int CDFOpenFile (file_spec, a_mode)
 #else
-static FILE *OpenFile (file_spec, a_mode)
+static FILE *CDFOpenFile (file_spec, a_mode)
 #endif
 char *file_spec;
 char *a_mode;                   
@@ -145,7 +157,7 @@ char *a_mode;
 }   
 
 /******************************************************************************
-* FindCache.
+* FindCache. If it is found, then move it to the head.
 ******************************************************************************/
 
 static vCACHE *FindCache (vFp, blockN)
@@ -176,6 +188,22 @@ long blockN;
   return NULL;
 }
 
+/******************************************************************************
+* CountCaches.
+******************************************************************************/
+
+static int CountCaches (vFp)
+vFILE *vFp;
+{
+  int ix = 0;
+  vCACHE *cache = vFp->cacheHead;
+  while (cache != NULL) {
+    ++ix;
+    cache = cache->next;
+  }
+  return ix;
+}
+
 /******************************************************************************
 * FlushCache64.
 * Write cache buffers to disk from the specified starting buffer to the last
@@ -536,54 +564,178 @@ vFILE *vFp;
   int ii;
 #endif
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-  int fh;
+  int fh = -1;
+#else
+  FILE *fp = NULL;
+#endif
+  int writable;
+#if defined(unix) || defined(vms)
+  int fileid;
+#endif
+#if defined(win32)
+  int err;
 #endif
   /****************************************************************************
   * Create the scratch file if necessary.  If so, the current file path is
   * actually the scratch directory to be used.
   ****************************************************************************/
+  if (pid == -1) {
+#   if defined(win32)
+      pid = (int) _getpid();
+#   else
+      pid = (int) getpid();
+#   endif
+    srand((unsigned int)pid*time(NULL));
+  }  
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
   if (vFp->fh == 0) {
 #else
   if (vFp->fp == NULL) {
 #endif
-    long i; char *tmpPath; size_t pathLength;
+    long i; char *tmpPath = NULL; size_t pathLength;
     pathLength = strlen(vFp->path) + 1 + 8 + 1 + EXT_LEN;
+    if (pathLength < DU_MAX_DIR_LEN) pathLength = DU_MAX_DIR_LEN;
     tmpPath = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
     if (tmpPath == NULL) return FALSE;
-    for (i = 1; i <= MAX_TMP; i++) {
-       strcpyX (tmpPath, vFp->path, 0);
-       AppendToDir (tmpPath, "");
-       snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
-                 "TMP%05ld.%s", i, vFp->scratchExt);
-       if (!IsReg(tmpPath)) {
-#if defined(win32) && (_FILE_OFFSET_BITS == 64)
-         fh = OpenFile (tmpPath, WRITE_PLUS_a_mode);
-         if (fh == -1) {
-#else
-	 FILE *fp = OpenFile (tmpPath, WRITE_PLUS_a_mode);
-	 if (fp == NULL) {
-#endif
-	   cdf_FreeMemory (tmpPath, NULL);
-	   return FALSE;
-	 }
-	 cdf_FreeMemory (vFp->path, NULL);
-	 vFp->path = tmpPath;
-#if defined(win32) && (_FILE_OFFSET_BITS == 64)
-	 vFp->fh = fh;
-#else
-	 vFp->fp = fp;
-#endif
-	 break;
-       }
+    if (vFp->path != NULL && strlen(vFp->path) != 0) { 
+      strcpyX (tmpPath, vFp->path, 0);
+      AppendToDir (tmpPath, "");
+#     if defined(unix) || defined(vms)
+        strcatX (tmpPath, TmpFileTemplate, 0);
+        fileid = mkstemp(tmpPath);
+        if (fileid != -1) fp = fdopen(fileid, "w+");
+        else {
+          for (i = 1; i <= MAX_TMP; i++) {
+            strcpyX (tmpPath, vFp->path, 0);
+            AppendToDir (tmpPath, "");
+            snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+            if (!IsReg(tmpPath)) {
+              fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+              if (fp != NULL) break;
+            }
+          }
+        }
+#     else
+#       if defined(win32)
+          fh = -1;
+          strcatX (tmpPath, TmpFileTemplate, 0);
+          err = _mktemp_s(tmpPath, (int)strlen(tmpPath)+1);
+          if (err == 0) {
+            fh = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+          } else {
+            if (err == EEXIST) {
+              int i1, i2;
+              for (i = 1; i <= MAX_TMP; i++) {
+                fh = -1;
+                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                strcpyX (tmpPath, vFp->path, 0);
+                AppendToDir (tmpPath, "");
+                snprintf (EofS(tmpPath), (size_t) pathLength + 1 -
+                                         strlen(tmpPath),
+                          "TMP%c%c%5.5d.%s", i1, i2, rand(), vFp->scratchExt);
+                if (!IsReg(tmpPath)) {
+                  fh = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+                  if (fh != -1) break;
+                }
+              }
+            }
+          }
+#       else
+          for (i = 1; i <= MAX_TMP; i++) {
+            strcpy (tmpPath, vFp->path);
+            AppendToDir (tmpPath, "");
+            snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+            if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+            if (fp != NULL) break;
+          }
+#       endif
+#     endif
+    } else {
+      char *tmpFolder = NULL;
+      strcpy (tmpPath, "");
+#     if defined(unix) || defined(vms)
+        tmpFolder = (char *) cdf_AllocateMemory ((size_t)pathLength + 1, NULL);
+        if (tmpFolder == NULL) return FALSE;
+        writable = access(".", W_OK);
+        if (writable == 0) {
+          strcpy (tmpPath, TmpFileTemplate);
+          strcpy (tmpFolder, "");
+        } else {
+#         if defined(unix)
+            strcpy (tmpPath, "/tmp/");
+            strcpy (tmpFolder, tmpPath);
+#         else
+            strcpy (tmpPath, "SYS$SCRATCH:");
+            strcpy (tmpFolder, tmpPath);
+#         endif
+          strcatX (tmpPath, TmpFileTemplate, 0);
+        }
+        fileid = mkstemp(tmpPath);
+        if (fileid != -1) fp = fdopen(fileid, "w+");
+        else {
+          for (i = 1; i <= MAX_TMP; i++) {
+            strcpy (tmpPath, tmpFolder);
+            snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                      "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+            if (!IsReg(tmpPath)) {
+              fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+              if (fp != NULL) break;
+            }
+          }
+        }
+        if (tmpFolder != NULL) cdf_FreeMemory (tmpFolder, NULL);
+#     else
+#       if defined(win32)
+          fh = -1;
+          strcatX (tmpPath, TmpFileTemplate, 0);
+          err = _mktemp_s(tmpPath, (int)strlen(tmpPath)+1 );
+          if (err == 0) {
+            fh = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+          } else {
+            if (err == EEXIST) {
+              int i1, i2;
+              for (i = 1; i <= MAX_TMP; i++) {
+                fh = -1;
+                strcpy (tmpPath, "");
+                i1 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                i2 = (int) (((double)rand() / (RAND_MAX + 1)) * 25) + (int) 'A';
+                snprintf (EofS(tmpPath), (size_t) pathLength + 1 -
+                                         strlen(tmpPath),
+                          "TMP%c%c%5.5d.%s", i1, i2, rand(), vFp->scratchExt);
+                if (!IsReg(tmpPath)) {
+                  if (_access(tmpPath, 0) != 0) {
+                    fh = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+                    if (fh != -1) break;
+                  }
+                }
+              }
+            }
+          }
+#       else
+          snprintf (EofS(tmpPath), (size_t) pathLength + 1 - strlen(tmpPath),
+                    "TMP%08ld.%s", (long)(rand()%100000000), vFp->scratchExt);
+          if (!IsReg(tmpPath)) fp = CDFOpenFile (tmpPath, WRITE_PLUS_a_mode);
+#       endif
+#     endif
     }
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
-    if (vFp->fh == -1) {
+    if (fh == -1) {
 #else
-    if (vFp->fp == NULL) {	   /* Hardly seems likely but we'll check... */
+    if (fp == NULL) {
 #endif
       cdf_FreeMemory (tmpPath, NULL);
       return FALSE;
+    } else {
+      cdf_FreeMemory (vFp->path, NULL);
+      vFp->path = tmpPath;
+#if defined(win32) && (_FILE_OFFSET_BITS == 64)
+      vFp->fh = fh;
+#else
+      vFp->fp = fp;
+#endif
     }
   }
   /****************************************************************************
@@ -670,7 +822,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 {
   size_t nBytesX;       /* Total number of bytes in buffer. */
   size_t nBytes;	/* Total number of bytes to read. */
-  OFF_T  remainingItems;	/* Number of items remaining after the offset. */
+  OFF_T  remainingItems;/* Number of items remaining after the offset. */
   size_t nItems;	/* Number of items to actually be read. */
   long firstBlockN;    	/* First block involved in read. */
   long lastBlockN;     	/* Last block involved in read. */
@@ -739,11 +891,6 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   ****************************************************************************/
   lastBlockN = (long) ((vFp->offset64 + (OFF_T) nBytes - 1) / 
 			(OFF_T) nCACHE_BUFFER_BYTEs); 
-/*
-  tmpBlockN = ((OFF_T) (vFp->offset64 + nBytes - 1)) / nCACHE_BUFFER_BYTEs;
-  if (tmpBlockN > maxCount) lastBlockN = 0;
-  else lastBlockN = (long) tmpBlockN;
-*/
   for (blockN = atBlockN; blockN <= lastBlockN; blockN++) {
      xBytes = (size_t) (((OFF_T)vFp->offset64) + (OFF_T) nBytes - 
 		        ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs);
@@ -779,6 +926,7 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
 	   return 0;
 	 }
        }
+
      }
      buffer = (Byte1 *) buffer + xBytes;
   }
@@ -831,11 +979,6 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   * the cache (even if a full block).
   ****************************************************************************/
   firstBlockN = (long) (vFp->offset64 / (OFF_T) nCACHE_BUFFER_BYTEs); 
-/*
-  tmpBlockN = (OFF_T) (vFp->offset64 / nCACHE_BUFFER_BYTEs);
-  if (tmpBlockN > maxCount) firstBlockN = 0;
-  else firstBlockN = (long) tmpBlockN; 
-*/
   bufferOffset = (int) ((OFF_T) vFp->offset64 % nCACHE_BUFFER_BYTEs);
   nBytesInBlock = nCACHE_BUFFER_BYTEs - bufferOffset;
   xBytes = MINIMUM (nBytes, nBytesInBlock);
@@ -888,11 +1031,6 @@ vFILE *vFp;             /* Pointer to vFILE structure. */
   ****************************************************************************/
   lastBlockN = (long) ((vFp->offset64 + (OFF_T) nBytes - 1) / 
 			(OFF_T) nCACHE_BUFFER_BYTEs); 
-/*  tmpBlockN = ((OFF_T) (((OFF_T)vFp->offset64) + nBytes - 1)) / 
-			nCACHE_BUFFER_BYTEs;
-  if (tmpBlockN > maxCount) lastBlockN = 0;
-  else lastBlockN = (long) tmpBlockN;
-*/
   for (blockN = atBlockN; blockN <= lastBlockN; blockN++) {
      xBytes = (size_t) (((OFF_T)vFp->offset64) + (OFF_T) nBytes - 
 		        ((OFF_T) blockN) * nCACHE_BUFFER_BYTEs);
@@ -992,7 +1130,7 @@ char *a_mode;           /* Access mode. */
   /****************************************************************************
   * Open the file.
   ****************************************************************************/
-  fp = OpenFile (file_spec, a_mode);
+  fp = CDFOpenFile (file_spec, a_mode);
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
   if (fp == -1) return NULL;
 #else
@@ -1045,7 +1183,7 @@ char *a_mode;           /* Access mode. */
 	FCLOSE64 (fp);
 	return NULL;
       }
-      fp = OpenFile (file_spec, a_mode);
+      fp = CDFOpenFile (file_spec, a_mode);
       if (fp == NULL) return NULL;
     }
   }
@@ -1271,18 +1409,16 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   ****************************************************************************/
 #if defined(win32) && (_FILE_OFFSET_BITS == 64)
   if (vFp->fh != 0) {
-    if (CDF != NULL && (!CDF->readOnly || CDF->status == READ_WRITE) &&
+    if (CDF != NULL && !(CDF->readOnly || CDF->status == READ_ONLY) &&
         CDF->singleFile && (CDF->checksum != NONE_CHECKSUM)) {
-/*      if (!FLUSHv64(vFp)) error = TRUE; */
       if (!CDFAddChecksum(CDF)) error = TRUE;
     }
     if (FCLOSE64 (vFp->fh) == EOF) error = TRUE;
   }
 #else
   if (vFp->fp != NULL) {
-    if (CDF != NULL && (!CDF->readOnly || CDF->status == READ_WRITE) && 
+    if (CDF != NULL && !(CDF->readOnly || CDF->status == READ_ONLY) && 
         CDF->singleFile && (CDF->checksum != NONE_CHECKSUM)) {
-/*      if (!FLUSHv64(vFp)) error = TRUE; */
       if (!CDFAddChecksum(CDF)) error = TRUE;
     }
     if (FCLOSE64 (vFp->fp) == EOF) error = TRUE;
@@ -1373,7 +1509,7 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
 #else
   if (vFp->fp != NULL) {
 #endif
-    if (!DeleteFile(vFp->path)) error = TRUE;
+    if (!CDFdeleteFile(vFp->path)) error = TRUE;
   }
   /****************************************************************************
   * Pass back statistics (if requested).
@@ -1400,3 +1536,24 @@ vSTATS *vStats;         /* Pointer to statistics structure. */
   return BOO(error,EOF,0);
 }
 
+
+/******************************************************************************
+* GetMyPID.
+* Returns the PID of the session. If -1 then it is not set.
+******************************************************************************/
+
+VISIBLE_PREFIX int GetMyPID ()
+{
+  return pid;
+}
+
+/******************************************************************************
+* SetMyPID.
+* Sets the PID of the session.
+******************************************************************************/
+
+VISIBLE_PREFIX void SetMyPID (int npid)
+{
+  pid = npid;
+}
+
